
!Multihistogram reweighting for the 2D Ising model
! by D.Jost 02/10/09

program mainmulti

implicit none

include 'global.var' !include global variables

real*8,allocatable		:: SpMtx(:,:) !current spin matrix
real*8,allocatable		:: N0(:,:,:) !histograms of visits
real*8,allocatable		:: Z(:) !partition function
real*8,allocatable		:: Temp(:) !scanned temperatures
real*8		::M !current magnetization
real*8		::NNC !current nearest-neighbour coupling

real*8		::tmm,randomnumber !other local variables
integer		::time,i,j !other local variables

!initialize random number generator
call random_seed()
call initializerandomnumbergenerator(tmm)

!open the input file
open(10,file='input.dat')
read(10,*) P !size of the spin matrix
read(10,*) NumberFlip !number of spin flips
read(10,*) Ntemp !number of runs at different temperatures
allocate(Temp(Ntemp)) 
read(10,*) (Temp(i),i=1,Ntemp) !corresponding temperatures
read(10,*) TempMin !minimal scanned temperature
read(10,*) TempMax !maximal scanned temperature
read(10,*) dTemp !temperature interval
close(10)

N=P*P

!allocate variables the histogram N0
allocate(N0(Ntemp,N+1,N+1)) !N0(i,:,:) is the histogram of visits for temperature Temp(i)
allocate(SpMtx(P,P))
allocate(Z(Ntemp)) ! Z(i) is the estimated partition function at temperature Temp(i)

!for every temperatures in Temp, we perform a importance sampling run
do i=1,Ntemp
	T0=Temp(i)
	
	!initialize variables
	N0(i,:,:)=0d0
	call randomconfig(SpMtx,M,NNC)

	!do the metropolis scheme
	do j=1,NumberFlip
		call metropolis(SpMtx,M,NNC,N0(i,:,:))
	end do
	
	!compute the current rmms magnetization
	call Mbar(N0(i,:,:))
end do

! compute the different Z(i) by iteration 
call equation(N0,Temp,Z)

!compute the density of state and the rms magnetization from TempMin to Tempmax
call density(N0,Temp,Z)

end program mainmulti